import 'dart:async';
import 'dart:convert';

import 'package:collection/collection.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutterx_application/flutterx_application.dart';
import 'package:flutterx_application/src/routing/webutils/webutils.dart'
    if (dart.library.html) 'package:flutterx_application/src/routing/webutils/webutils_html.dart';
import 'package:flutterx_application/src/ui/system_ui_style.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

part 'app_route_activity.dart';
part 'app_route_bottom_sheet.dart';
part 'app_route_dialog.dart';
part 'app_route_information_parser.dart';
part 'app_route_information_provider.dart';
part 'app_router_delegate.dart';

/// Arguments that can be passed to a route
typedef RouteArgs = Map<String, dynamic>;

/// Builder of a route
typedef RouteWidgetBuilder = Widget Function(BuildContext context, RouteArgs args);

/// Builder of a modal route with additional output arguments
typedef ModalRouteWidgetBuilder = Widget Function(BuildContext context, RouteArgs args, RouteArgs outArgs);

/// Builder of a page title
typedef RouteTitleBuilder = String Function(RouteArgs args);

/// Builder of a page key
typedef RouteKeyBuilder = LocalKey? Function(RouteArgs args);

/// A route to a navigation UI component.
///
/// [location] path of this route
/// [T] the result type for this route
@immutable
abstract class AppRoute<T> {
  final String location;

  const AppRoute._({required this.location});

  /// Open this route with given [args] on top of the current or replace current if [replace] is true
  Future<T> open(BuildContext context, {RouteArgs args = const {}});

  Future<T> openReplace(BuildContext context, {RouteArgs args = const {}, Object? result}) {
    Navigator.of(context).pop(result);
    return open(context, args: args);
  }

  @override
  bool operator ==(Object other) => other is AppRoute && other.location == location;

  @override
  int get hashCode => hashValues(runtimeType, location);

  @override
  String toString() => '$runtimeType($location)';

  static String routeName(Object data) => data is String ? data : data.toString().snakeCase;

  static String routeLocation(Object data) => Navigator.defaultRouteName + routeName(data);
}
